Index: trunk/source/global/management/include/G4Allocator.hh
===================================================================
--- trunk/source/global/management/include/G4Allocator.hh	(revision 1058)
+++ trunk/source/global/management/include/G4Allocator.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4Allocator.hh,v 1.18 2006/06/29 19:01:16 gunter Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4Allocator.hh,v 1.19 2009/10/29 16:01:28 gcosmo Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -99,5 +99,5 @@
       // Returns the address of values
 
-    pointer allocate(size_type n, void* hint = 0)
+    pointer allocate(size_type n, void* = 0)
     {
       // Allocates space for n elements of type Type, but does not initialise
Index: trunk/source/global/management/include/G4ErrorPropagatorData.hh
===================================================================
--- trunk/source/global/management/include/G4ErrorPropagatorData.hh	(revision 1058)
+++ trunk/source/global/management/include/G4ErrorPropagatorData.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4ErrorPropagatorData.hh,v 1.3 2007/06/08 10:33:47 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4ErrorPropagatorData.hh,v 1.5 2009/05/19 13:31:47 gcosmo Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 //
@@ -38,5 +38,4 @@
 //   and manager verbosity for the error propagation classes.
 
-// History:
 // - Created. P.Arce, 2004.
 // --------------------------------------------------------------------
@@ -56,4 +55,8 @@
                     G4ErrorState_TargetCloserThanBoundary,
                     G4ErrorState_StoppedAtTarget };
+
+enum G4ErrorStage  { G4ErrorStage_Inflation = 1,
+                     G4ErrorStage_Deflation };
+
 class G4ErrorTarget;
 
@@ -72,4 +75,7 @@
   G4ErrorState GetState() const;
   void SetState( G4ErrorState sta );
+
+  G4ErrorStage GetStage() const;
+  void SetStage( G4ErrorStage sta );
 
   const G4ErrorTarget* GetTarget( G4bool mustExist = 0) const;
@@ -94,4 +100,6 @@
   G4ErrorState theState;
 
+  G4ErrorStage theStage;
+
   G4ErrorTarget* theTarget;
 
Index: trunk/source/global/management/include/G4ErrorPropagatorData.icc
===================================================================
--- trunk/source/global/management/include/G4ErrorPropagatorData.icc	(revision 1058)
+++ trunk/source/global/management/include/G4ErrorPropagatorData.icc	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4ErrorPropagatorData.icc,v 1.4 2007/06/08 10:33:47 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4ErrorPropagatorData.icc,v 1.5 2009/05/14 13:46:40 arce Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 //
@@ -59,4 +59,16 @@
 
 inline 
+void G4ErrorPropagatorData::SetStage( G4ErrorStage sta )
+{
+  theStage = sta;
+}
+
+inline 
+G4ErrorStage G4ErrorPropagatorData::GetStage() const
+{
+  return theStage;
+}
+
+inline 
 const G4ErrorTarget* G4ErrorPropagatorData::GetTarget( G4bool mustExist ) const
 {
Index: trunk/source/global/management/include/G4LPhysicsFreeVector.icc
===================================================================
--- trunk/source/global/management/include/G4LPhysicsFreeVector.icc	(revision 1058)
+++ trunk/source/global/management/include/G4LPhysicsFreeVector.icc	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4LPhysicsFreeVector.icc,v 1.13 2008/09/22 08:26:33 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4LPhysicsFreeVector.icc,v 1.14 2009/06/25 10:05:26 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -52,5 +52,5 @@
    if(binNumber == 0)
      { edgeMin = binValue; }
-   else if( numberOfBin - 1 == binNumber)
+   else if( numberOfNodes - 1 == binNumber)
      { edgeMax = binValue; }
 }
@@ -84,6 +84,6 @@
 {
    G4int n1 = 0;
-   G4int n2 = numberOfBin/2;
-   G4int n3 = numberOfBin - 1;
+   G4int n2 = numberOfNodes/2;
+   G4int n3 = numberOfNodes - 1;
    while (n1 != n3 - 1)
    {
Index: trunk/source/global/management/include/G4PhysicsFreeVector.hh
===================================================================
--- trunk/source/global/management/include/G4PhysicsFreeVector.hh	(revision 1058)
+++ trunk/source/global/management/include/G4PhysicsFreeVector.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4PhysicsFreeVector.hh,v 1.12 2008/09/22 14:49:57 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4PhysicsFreeVector.hh,v 1.13 2009/06/25 10:05:26 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -108,5 +108,5 @@
 
   size_t lowerBound = 0;
-  size_t upperBound = numberOfBin-1;
+  size_t upperBound = numberOfNodes-2;
 
   while (lowerBound <= upperBound)
Index: trunk/source/global/management/include/G4PhysicsOrderedFreeVector.hh
===================================================================
--- trunk/source/global/management/include/G4PhysicsOrderedFreeVector.hh	(revision 1058)
+++ trunk/source/global/management/include/G4PhysicsOrderedFreeVector.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4PhysicsOrderedFreeVector.hh,v 1.11 2008/09/22 14:49:57 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4PhysicsOrderedFreeVector.hh,v 1.12 2009/06/25 10:05:26 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 ////////////////////////////////////////////////////////////////////////
@@ -86,5 +86,5 @@
         ////////////
 
-	void InsertValues(G4double energy, G4double value); 
+  	void InsertValues(G4double energy, G4double value); 
 
 	G4double GetLowEdgeEnergy(size_t binNumber) const;
Index: trunk/source/global/management/include/G4PhysicsOrderedFreeVector.icc
===================================================================
--- trunk/source/global/management/include/G4PhysicsOrderedFreeVector.icc	(revision 1058)
+++ trunk/source/global/management/include/G4PhysicsOrderedFreeVector.icc	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4PhysicsOrderedFreeVector.icc,v 1.8 2006/06/29 19:02:31 gunter Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4PhysicsOrderedFreeVector.icc,v 1.9 2009/06/25 10:05:26 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 ////////////////////////////////////////////////////////////////////////
@@ -73,5 +73,5 @@
 void G4PhysicsOrderedFreeVector::DumpValues()
 {
-   for (size_t i = 0; i < numberOfBin; i++)
+   for (size_t i = 0; i < numberOfNodes; i++)
    {
       G4cout << binVector[i] << "\t" << dataVector[i] << G4endl;
@@ -83,6 +83,6 @@
 {
    G4int n1 = 0;
-   G4int n2 = numberOfBin/2;
-   G4int n3 = numberOfBin - 1;
+   G4int n2 = numberOfNodes/2;
+   G4int n3 = numberOfNodes - 1;
    while (n1 != n3 - 1)
    {
Index: trunk/source/global/management/include/G4PhysicsVector.icc
===================================================================
--- trunk/source/global/management/include/G4PhysicsVector.icc	(revision 1058)
+++ trunk/source/global/management/include/G4PhysicsVector.icc	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4PhysicsVector.icc,v 1.17 2008/09/22 08:26:33 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4PhysicsVector.icc,v 1.23 2009/11/18 11:42:03 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -59,8 +59,16 @@
 //---------------------------------------------------------------
 
+inline
+ G4double G4PhysicsVector::Energy(const size_t binNumber) const
+{
+  return binVector[binNumber];
+}
+
+//---------------------------------------------------------------
+
 inline 
  size_t G4PhysicsVector::GetVectorLength() const
 {
-  return numberOfBin;
+  return numberOfNodes;
 }
 
@@ -94,11 +102,13 @@
   // numberOfBin-1. 
 
-  if( !secDerivative ) { FillSecondDerivatives(); }
+  if(0 == secDerivative.size() ) { FillSecondDerivatives(); }
 
   // check bin value
-  G4double delta = binVector[lastBin+1] - binVector[lastBin];
-
-  G4double a = (binVector[lastBin+1] - lastEnergy)/delta;
-  G4double b = (lastEnergy - binVector[lastBin])/delta;
+  G4double x1 = binVector[lastBin];
+  G4double x2 = binVector[lastBin+1];
+  G4double delta = x2 - x1;
+
+  G4double a = (x2 - lastEnergy)/delta;
+  G4double b = (lastEnergy - x1)/delta;
    
   // Final evaluation of cubic spline polynomial for return   
@@ -107,6 +117,6 @@
 
   G4double res = a*y1 + b*y2 + 
-        ((a*a*a - a)*secDerivative[lastBin] +
-         (b*b*b - b)*secDerivative[lastBin+1])*delta*delta/6.0  ;
+        ( (a*a*a - a)*secDerivative[lastBin] +
+          (b*b*b - b)*secDerivative[lastBin+1] )*delta*delta/6.0;
 
   return res;
@@ -117,4 +127,12 @@
 inline
  G4double G4PhysicsVector::GetValue(G4double theEnergy, G4bool&) 
+{
+  return Value(theEnergy);
+}
+
+//---------------------------------------------------------------
+
+inline
+ G4double G4PhysicsVector::Value(G4double theEnergy) 
 {
   // Use cache for speed up - check if the value 'theEnergy' is same as the 
@@ -134,16 +152,12 @@
      lastValue  = dataVector[0];
 
-  } else if(theEnergy < lastEnergy && theEnergy >= binVector[lastBin-1]) {
-     lastBin--;
-     lastEnergy = theEnergy;
-     Interpolation();
-
   } else if( theEnergy >= edgeMax ){
-     lastBin = numberOfBin-1;
+     lastBin = numberOfNodes-2;
      lastEnergy = edgeMax;
-     lastValue  = dataVector[lastBin];
+     lastValue  = dataVector[numberOfNodes-1];
 
   } else {
      lastBin = FindBinLocation(theEnergy); 
+     if(lastBin >= numberOfNodes-1) {lastBin = numberOfNodes-2;} 
      lastEnergy = theEnergy;
      Interpolation();
@@ -167,9 +181,4 @@
 {
   dataVector[binNumber] = theValue;
-
-  // Fill the bin which is hidden to user with theValue. This is to 
-  // handle correctly when Energy=theEmax in getValue.
-
-  if(binNumber==numberOfBin-1) { dataVector[binNumber+1] = theValue; }
 }
 
@@ -181,5 +190,5 @@
   G4bool status=false;
 
-  if(numberOfBin > 0) { status=true; }
+  if(numberOfNodes > 0) { status=true; }
   return status;
 }
Index: trunk/source/global/management/include/G4String.hh
===================================================================
--- trunk/source/global/management/include/G4String.hh	(revision 1058)
+++ trunk/source/global/management/include/G4String.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4String.hh,v 1.10 2008/12/08 14:16:05 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4String.hh,v 1.15 2009/08/10 10:18:09 gcosmo Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -78,7 +78,7 @@
   inline G4int operator!() const;
 
-  inline G4bool operator==(G4String) const;
+  inline G4bool operator==(const G4String&) const;
   inline G4bool operator==(const char*) const;
-  inline G4bool operator!=(G4String) const;
+  inline G4bool operator!=(const G4String&) const;
   inline G4bool operator!=(const char*) const;
 
@@ -138,5 +138,4 @@
   inline G4bool operator!=(const char*) const;
 
-  //inline G4String operator () (unsigned int, unsigned int);
   inline operator const char*() const;
   inline G4SubString operator()(str_size, str_size);
@@ -160,5 +159,5 @@
   inline G4int last(char) const;
 
-  inline G4bool contains(std::string) const;
+  inline G4bool contains(const std::string&) const;
   inline G4bool contains(char) const;
 
@@ -184,9 +183,4 @@
   inline unsigned int hash( caseCompare cmp = exact ) const;
   inline unsigned int stlhash() const;
-
-  // useful for supplying hash functions to template hash collection ctors
-  //
-  static inline unsigned int hash(const G4String&);
-
 };
 
Index: trunk/source/global/management/include/G4String.icc
===================================================================
--- trunk/source/global/management/include/G4String.icc	(revision 1058)
+++ trunk/source/global/management/include/G4String.icc	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4String.icc,v 1.11 2008/12/08 14:16:05 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4String.icc,v 1.19 2009/08/10 10:30:03 gcosmo Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -91,25 +91,25 @@
 G4int G4SubString::operator!() const
 {
-  return extent==0 ? 1 : 0;
-}
-
-G4bool G4SubString::operator==(G4String s) const
-{
-  return mystring->substr(mystart,extent) == s ? true:false;
+  return (extent==0) ? 1 : 0;
+}
+
+G4bool G4SubString::operator==(const G4String& s) const
+{
+  return (mystring->find(s,mystart,extent) != std::string::npos);
 }
 
 G4bool G4SubString::operator==(const char* s) const
 {
-  return mystring->substr(mystart,extent) == std::string(s) ? true:false;
-}
-
-G4bool G4SubString::operator!=(G4String s) const
-{
-  return mystring->substr(mystart,extent) != std::string(s) ? true:false;
+  return (mystring->find(s,mystart,extent) != std::string::npos);
+}
+
+G4bool G4SubString::operator!=(const G4String& s) const
+{
+  return (mystring->find(s,mystart,extent) == std::string::npos);
 }
 
 G4bool G4SubString::operator!=(const char* s) const
 {
-  return mystring->substr(mystart,extent) != std::string(s) ? true:false;
+  return (mystring->find(s,mystart,extent) == std::string::npos);
 }
 
@@ -126,5 +126,5 @@
 G4bool G4SubString::isNull() const
 {
-  return extent==0 ? true : false;
+  return (extent==0);
 }
 
@@ -219,28 +219,20 @@
 G4bool G4String::operator==(const G4String &s) const
 {
-  const std_string *a=this;
-  const std_string *b=&s;
-  return *a==*b;
+  return (std_string::compare(s) == 0);
 }
 
 G4bool G4String::operator==(const char* s) const
 {
-  const std_string *a=this;
-  const std_string b=s;
-  return *a==b;
+  return (std_string::compare(s) == 0);
 }
 
 G4bool G4String::operator!=(const G4String &s) const
 {
-  const std_string *a=this;
-  const std_string *b=&s;
-  return *a!=*b;
+  return !(*this == s);
 }
 
 G4bool G4String::operator!=(const char* s) const
 {
-  const std_string *a=this;
-  const std_string b=s;
-  return *a!=b;
+  return !(*this == s);
 }
 
@@ -268,16 +260,12 @@
 G4int G4String::compareTo(const char* s, caseCompare mode)
 {
-  if(mode==exact)
-    { return strcmp(c_str(),s); }
-  else
-    { return strcasecompare(c_str(),s); }
+  return (mode==exact) ? strcmp(c_str(),s)
+                       : strcasecompare(c_str(),s);
 }
 
 G4int G4String::compareTo(const G4String& s, caseCompare mode)
 {
-  if(mode==exact)
-    { return strcmp(c_str(),s.c_str()); }
-  else
-    { return strcasecompare(c_str(),s.c_str()); }
+  return (mode==exact) ? strcmp(c_str(),s.c_str())
+                       : strcasecompare(c_str(),s.c_str());
 }
 
@@ -298,10 +286,12 @@
 {
   char tmp[1024];
-  if ( skipWhite ) {
+  if ( skipWhite )
+  {
     s >> std::ws;
     s.getline(tmp,1024);
     *this=tmp;
   }
-  else {
+  else
+  {
     s.getline(tmp,1024);    
     *this=tmp;
@@ -325,6 +315,5 @@
 G4String& G4String::remove(str_size n)
 {
-  if(n<size())
-    { erase(n,size()-n); }
+  if(n<size()) { erase(n,size()-n); }
   return *this;
 }
@@ -346,20 +335,12 @@
 }
 
-G4bool G4String::contains(std::string s) const
-{
-  G4int i=std_string::find(s);
-  if(i != G4int(std_string::npos))
-    { return true; }
-  else
-    { return false; }
+G4bool G4String::contains(const std::string& s) const
+{
+  return (std_string::find(s) != std_string::npos);
 }
 
 G4bool G4String::contains(char c) const
 {
-  G4int i=std_string::find(c);
-  if(i != G4int(std_string::npos))
-    { return true; }
-  else
-    { return false; }
+  return (std_string::find(c) != std_string::npos);
 }
 
@@ -474,7 +455,2 @@
   return str_size(h);
 }
-
-unsigned int G4String::hash(const G4String& s)
-{
-  return s.hash();
-}
Index: trunk/source/global/management/include/G4Version.hh
===================================================================
--- trunk/source/global/management/include/G4Version.hh	(revision 1058)
+++ trunk/source/global/management/include/G4Version.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4Version.hh,v 1.17 2008/12/02 09:15:58 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4Version.hh,v 1.24 2009/06/19 14:29:09 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // Version information
@@ -47,9 +47,9 @@
 
 #ifndef G4VERSION_NUMBER
-#define G4VERSION_NUMBER  920
+#define G4VERSION_NUMBER  930
 #endif
 
 #ifndef G4VERSION_TAG
-#define G4VERSION_TAG "$Name: geant4-09-02-ref-02 $"
+#define G4VERSION_TAG "$Name: global-V09-02-10 $"
 #endif
 
@@ -58,6 +58,6 @@
 #include "G4String.hh"
 
-static const G4String G4Version = "$Name: geant4-09-02-ref-02 $";
-static const G4String G4Date    = "(19-December-2008)";
+static const G4String G4Version = "$Name: global-V09-02-10 $";
+static const G4String G4Date    = "(5-June-2009)";
 
 #endif

Index: trunk/source/global/management/include/G4PhysicsVector.hh
===================================================================
--- trunk/source/global/management/include/G4PhysicsVector.hh	(revision 1058)
+++ trunk/source/global/management/include/G4PhysicsVector.hh	(revision 1193)
@@ -25,6 +25,6 @@
 //
 //
-// $Id: G4PhysicsVector.hh,v 1.18 2008/09/22 08:26:33 gcosmo Exp $
-// GEANT4 tag $Name: geant4-09-02-ref-02 $
+// $Id: G4PhysicsVector.hh,v 1.25 2009/11/04 11:32:43 vnivanch Exp $
+// GEANT4 tag $Name: global-V09-02-10 $
 //
 // 
@@ -51,4 +51,6 @@
 //    09 Mar. 2001, H.Kurashige : Added G4PhysicsVectorType & Store/Retrieve()
 //    02 Apr. 2008, A.Bagulya : Added SplineInterpolation() and SetSpline()
+//    11 May  2009, V.Ivanchenko : Added ComputeSecondDerivatives
+//    19 Jun. 2009, V.Ivanchenko : Removed hidden bin 
 //
 //---------------------------------------------------------------
@@ -84,38 +86,71 @@
          // destructor
 
-    inline G4double GetValue(G4double theEnergy, G4bool& isOutRange);
+    inline G4double Value(G4double theEnergy);
          // Get the cross-section/energy-loss value corresponding to the
          // given energy. An appropriate interpolation is used to calculate
          // the value. 
-         // [Note] isOutRange is not used anymore. This argument is kept
-         //        for the compatibility reason.
+
+    inline G4double GetValue(G4double theEnergy, G4bool& isOutRange);
+         // Obolete method to get value, isOutRange is not used anymore. 
+         // This method is kept for the compatibility reason.
 
     G4int operator==(const G4PhysicsVector &right) const ;
     G4int operator!=(const G4PhysicsVector &right) const ;
+
     inline G4double operator[](const size_t binNumber) const ;
          // Returns simply the value in the bin specified by 'binNumber'
-         // of the dataVector. The boundary check will be Done. If you
-         // don't want this check, use the operator ().
+         // of the dataVector. The boundary check will not be done. 
+
     inline G4double operator()(const size_t binNumber) const ;
          // Returns simply the value in the bin specified by 'binNumber'
-         // of the dataVector. The boundary check will not be Done. If 
-         // you want this check, use the operator [].
-
-    inline void PutValue(size_t binNumber, G4double theValue);
+         // of the dataVector. The boundary check will not be Done. 
+
+    inline void PutValue(size_t index, G4double theValue);
          // Put 'theValue' into the bin specified by 'binNumber'.
          // Take note that the 'binNumber' starts from '0'.
          // To fill the vector, you have beforehand to Construct a vector
          // by the constructor with Emin, Emax, Nbin. 'theValue' should
-         // be the crosssection/energyloss value corresponding to the low 
-         // edge energy of the bin specified by 'binNumber'. You can get
-         // the low edge energy value of a bin by GetLowEdgeEnergy().
+         // be the crosssection/energyloss value corresponding to the  
+         // energy of the index. You can get this energy by the next method
+         // or by the old method GetLowEdgeEnergy().
+
+    void ScaleVector(G4double factorE, G4double factorV);
+         // Scale all values of the vector and second derivatives
+         // by factorV, energies by vectorE. This method may be applied 
+         // for example after Retrieve a vector from an external file to 
+         // convert values into Geant4 units
+
+    inline G4double Energy(size_t index) const;
+         // Returns simply the value in the energy specified by 'index'
+         // of the energy vector. The boundary check will not be done. 
+         // Use this function when you fill physis vector by PutValue().
+
     virtual G4double GetLowEdgeEnergy(size_t binNumber) const;
+         // Obsolete method
          // Get the energy value at the low edge of the specified bin.
          // Take note that the 'binNumber' starts from '0'.
-         // This value is defined when a physics vector is constructed
-         // by a constructor of a derived class. Use this function
-         // when you fill physis vector by PutValue().
+         // This value should be defined before the call.
+         // The boundary check will not be done.
+
     inline size_t GetVectorLength() const;
          // Get the toal length (bin number) of the vector. 
+
+    void FillSecondDerivatives();
+        // Initialise second derivatives for spline keeping 
+        // 3d derivative continues - default algorithm
+
+    void ComputeSecDerivatives();
+         // Initialise second derivatives for spline using algorithm 
+         // which garantee only 1st derivative continues 
+         // Warning: this method should be called when the vector 
+         // is already filled
+
+    void ComputeSecondDerivatives(G4double firstPointDerivative, 
+                                  G4double endPointDerivative);
+         // Initialise second derivatives for spline using 
+         // user defined 1st derivatives at edge points
+         // Warning: this method should be called when the vector 
+         // is already filled
+
     inline G4bool IsFilledVectorExist() const;
          // Is non-empty physics vector already exist?
@@ -124,4 +159,5 @@
          // Put a comment to the G4PhysicsVector. This may help to check
          // whether your are accessing to the one you want. 
+
     inline const G4String& GetComment() const;
          // Retrieve the comment of the G4PhysicsVector.
@@ -154,7 +190,8 @@
     G4PhysicsVectorType type;   // The type of PhysicsVector (enumerator)
 
-    G4double edgeMin;           // Lower edge value of the lowest bin
-    G4double edgeMax;           // Lower edge value of the highest bin
-    size_t numberOfBin;
+    G4double edgeMin;           // Energy of first point
+    G4double edgeMax;           // Energy of the last point
+
+    size_t numberOfNodes;
 
     G4double lastEnergy;        // Cache the last input value
@@ -163,7 +200,10 @@
 
     G4PVDataVector dataVector;    // Vector to keep the crossection/energyloss
-    G4PVDataVector binVector;     // Vector to keep the low edge value of bin
+    G4PVDataVector binVector;     // Vector to keep energy
+    G4PVDataVector secDerivative; // Vector to keep second derivatives 
 
   private:
+
+    G4bool SplinePossible();
 
     inline G4double LinearInterpolation();
@@ -174,9 +214,4 @@
     inline void Interpolation();
 
-    void FillSecondDerivatives();
-      // Initialise second derivatives for spline
-
-    G4double*  secDerivative;
-
     G4String   comment;
     G4bool     useSpline;
@@ -84,38 +86,71 @@
          // Put 'theValue' into the bin specified by 'binNumber'.
-         // Take note that the 'binNumber' starts from '0'.
-         // To fill the vector, you have beforehand to Construct a vector
+         // Take note that the 'index' starts from '0'.
+         // To fill the vector, you have beforehand to construct a vector
          // by the constructor with Emin, Emax, Nbin. 'theValue' should